<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Teknisa\Model\Geral\Entities;
use Util\Utilities;
use Util\InstanceProvider;

class CallComment
{
    /** @var \Doctrine\ORM\EntityManager $entityManager */
    protected $entityManager;
    /** @var \Util\Utilities*/
    private $environment;
    /** @var \Service\Integration\PMIntegration $pmIntegrationService */
    protected $pmIntegrationService;
    /** @var  \Factory\CallComment $callCommentFactory */
    protected $callCommentFactory;
    /** @var \Factory\EvolCont $evolContFactory */
    protected $evolContFactory;

    /** @var \Service\Integration\INTRANETIntegration $intranetIntegrationService */
    protected $intranetIntegrationService;
    /** @var \Teknisa\Model\Crm\Repositories\Contato $contatoRepository; */
    protected $contatoRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Operador $operadorRepository; */
    protected $operadorRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Paramgeral $paramGeralRepository */
    protected $paramGeralRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\CpCall $callRepository; */
    protected $callRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\CpCallComment $callCommentRepository; */
    protected $callCommentRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\CpCallAttachment $callAttachmentRepository */
    protected $callAttachmentRepository;

    public function __construct(EntityManager $entityManager,\Util\Utilities $environment, \Service\Integration\PMIntegration $pmIntegrationService,
                                \Factory\CallComment $callCommentFactory, \Factory\EvolCont $evolContFactory,
                                \Service\Integration\INTRANETIntegration $intranetIntegrationService)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->pmIntegrationService = $pmIntegrationService;
        $this->callCommentFactory = $callCommentFactory;
        $this->evolContFactory = $evolContFactory;
        $this->intranetIntegrationService = $intranetIntegrationService;

        $this->contatoRepository = $this->entityManager->getRepository('\Teknisa\Model\Crm\Entities\Contato');
        $this->operadorRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador');
        $this->paramGeralRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Paramgeral');
        $this->callRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpCall');
        $this->callCommentRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpCallComment');

        $this->callAttachmentRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpCallAttachment');
    }

    public function saveSupportCallComment($description, $createdBy, $callRelationship,$idevolinterna = 'N',
                                           $idsenevocont = 'S',$idrespnota = 'N') {
        $connection = $this->entityManager->getConnection();
        $commentTicketIntegration = null;
        try {
            $connection->beginTransaction();

            /** @var \Teknisa\Model\Geral\Entities\Operador $oper */
            $oper = $this->operadorRepository->findOneBy(array('cdoperador' => $createdBy));

            $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
            $supportCall =  $cpCallRepository->findOneBy(array('callRelationship' => $callRelationship));
            if(!$supportCall){
                /** @var \Teknisa\Model\Crm\Entities\Contato $contato */
                $contato = $this->contatoRepository->findOneBy(array('nrseqcontato' => $callRelationship));
                $call = InstanceProvider::getFactory()->getCall()->createProductCall($contato->getDsrefecont(), $contato->getCdoperinclusao(),
                    $contato->getDsaberturacaso(), Call::TYPE_TICKET, $callRelationship, $contato->getCdproduto());

                $callRelationship = $contato->getNrcontato();
                $this->entityManager->persist($call);
                $this->entityManager->flush();
            }

            $nrorg = $oper->getNrorg();
            $arrayCommentTicket[] = $this->callCommentFactory->createArrayCommentTicket($callRelationship,$createdBy,$nrorg,
                $description,$idevolinterna, $idsenevocont, $idrespnota);
            $commentTicketIntegration = Integration\CRMIntegration::addCommentTicketFromCRM($arrayCommentTicket);

            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $commentTicketIntegration;
    }

    public function findSupportCallComments($callRelationship)
    {
        $commnets = $this->callCommentRepository->getSupportCallCommentsFromCRM($callRelationship);
        foreach ($commnets as &$commnet) {
            $description = strip_tags($commnet['DESCRIPTION']);
            $commnet['DESCRIPTION'] = str_replace("&nbsp;", '', $description);
        }

        return $commnets;
    }

    public function saveProductCallComment($description, $createdBy = '', $callId, $subject = 'Adicionou novo comentário', $callRelationship = null, $callType = null)
    {
        $connection = $this->entityManager->getConnection();
        try {

            $connection->beginTransaction();
            $createdBy = $this->environment->getCurrentUserId();
            if (!$callId && $callRelationship) {
                if($callType == Call::TYPE_ISSUE){
                    $issue = $this->callRepository->findIssueById($callRelationship);
                    $productCall = InstanceProvider::getService()->getCall()->saveNewIssueTypeProductCall($callType, $issue['REFERENCE'], $issue['PRODUCT'],
                    $issue['CLASSIFICATION'], $issue['BOARD'], $issue['DESCRIPTION'], null, array(), $callRelationship);
                } else if ($callType == Call::TYPE_RICH_CLIENT){
                    $userId           = $this->environment->getCurrentUserId();
                    $situationFilters = InstanceProvider::getService()->getCall()->getAllSituationDividedByType();
                    $soliserv         = $this->callRepository->findSoliservById($callRelationship, $situationFilters, $userId);
                    $descriptionInTextPlain = Utilities::rtf2textPlain($soliserv['DESCRIPTION']);
                    $productCall = InstanceProvider::getService()->getCall()->
                                    saveNewRichClientTypeProductCall($callType, $soliserv['REFERENCE'],
                                    $soliserv['PRODUCT'], $soliserv['PRODUCT_TYPE'], $soliserv['PRODUCT_VERSION'],
                                    $soliserv['USE_LICENSE'], $soliserv['CLASSIFICATION'], $descriptionInTextPlain, $soliserv['REPORTER_BY'],
                                    array(), $soliserv['PRODUCT_RELEASE'], $soliserv['PRODUCT_PATCH'], $callRelationship);



                }

                $callId = $productCall['ID'];
            }

            $productCallComment = $this->callCommentFactory->createCallComment($description, $callId);
            $commentDate = '';
            $productCall = $this->callRepository->find($callId);
            $callRelationship = $productCall->getCallRelationship();
            /** @var Entities\Operador $commenter */
            $commenter = $this->operadorRepository->findOneBy(array('cdoperador' => $createdBy));

            if($productCall->getType() == Call::TYPE_ISSUE) {
                $this->pmIntegrationService->saveCommentOnIssueByProductCallComment($callRelationship, $description, $createdBy, $commenter->getCdCliente());
            } else if($productCall->getType() == Call::TYPE_RICH_CLIENT) {

                $nrorgId = $this->environment->getCurrentOrganizationId();
                $commentDate = $this->intranetIntegrationService->saveCommentOnSolicitacaoByProductCallComment($callRelationship, $subject, $description, $createdBy,$nrorgId);

                $comments = $this->callCommentRepository->getProductCallCommentsFromSolicitacao($callRelationship,null,true);
                if($comments && !empty($comments[1])){
                    $newFilename = $comments[0]['ORDERBY'];
                    $filename = $comments[1]['ORDERBY'];
                    $cdClienteBase = \Util\Utilities::getCdClienteBase();
                    $fileDir = $this->intranetIntegrationService->basicIntranetFileDir;
                    $folders    = $this->callAttachmentRepository->getAllAttachmentsFolder($callRelationship,
                    $fileDir, $cdClienteBase, true, $filename);

                    if($folders){
                        foreach($folders as $folder){
                            $newFolder = str_replace($filename, $newFilename, $folder['DIR']);
                            if(file_exists($folder['DIR'])){
                                rename($folder['DIR'], $newFolder);
                            }
                        }
                    }
                }
            }

            $this->entityManager->persist($productCallComment);
            $this->entityManager->flush();

            $connection->commit();

            return $commentDate;
        } catch(\Exception $e) {
            $connection->rollBack();
            throw new CustomException('ERR_NOT_POSSIBLE_TO_CREATE_COMMENT', 500, array( 'error' => $e->getMessage()));
        }
    }

    public function findCallComments( $callRelationship, $callType,$lastComment = false)
    {
        $callComments = array();
        if($callType == \Service\Call::TYPE_ISSUE || $callType == \Service\Call::TYPE_RICH_CLIENT){
            $callComments = $this->findProductCallComments($callRelationship, $callType);
        } else if ($callType == \Service\Call::TYPE_TICKET){
            $callComments = $this->findSupportCallComments( $callRelationship);
        }

        if($lastComment && empty($callComments)){
            Utilities::getCurrentTimeStatic()->format(\Zeedhi\DevOps\Util\Utilities::FORMAT_DATE_DEFAULT);
            $callComments = array(
                array(
                    "NAME" => "Mensagem de análise e aprovação",
                    "DESCRIPTION" => "Boa tarde. Segue especificação e orçamento para análise e aprovação.",
                    "CDFASETRAM" => "999",
                    "CREATED_BY_NAME" => "Teknisa",
                    "CREATED_BY" => "000000000997",
                    "CREATED_AT" => Utilities::getCurrentTimeStatic()->format(\Zeedhi\DevOps\Util\Utilities::FORMAT_DATE_DEFAULT),
                    "ORDERBY" => "000000000997"
                )
            );
        }

        return $callComments;
    }

    public function findProductCallComments($callRelationship, $callType)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        $productCall = $this->callRepository->findOneBy(array('callRelationship' => $callRelationship));

        $ticket = empty($productCall) ? null : $productCall->getTicketRelationship();
        $userId = $this->environment->getCurrentUserId();
        $comments = [];

        if($callType == \Service\Call::TYPE_ISSUE) {
            $comments = $this->pmIntegrationService->findCommentsOnIssueForProductCall($callRelationship);
        } else if($callType == \Service\Call::TYPE_RICH_CLIENT) {
            $situationFilters = InstanceProvider::getService()->getCall()->getAllSituationDividedByType();

            $productCall = $this->callRepository->findSoliservById($callRelationship, $situationFilters, $userId);
            if(!empty($productCall)){
                $comments = $this->intranetIntegrationService->findCommentsOnSolicitacaoForProductCall($callRelationship,
                    $this->environment->getCurrentUser());
            }
        }

        if($ticket){
            $ticketComments = $this->findSupportCallComments($ticket);
            $comments = array_merge($comments,$ticketComments);
        }

        return $comments;
    }

}
